from os import path

class ConfigPath(object):
    def __init__(self):
        self.config_paths = [path.abspath(path.curdir),
                             path.join(path.expanduser('~'), '.magpie'),
                             path.dirname(__file__)]

    def __getattr__(self, key):
        for p in self.config_paths:
            fname = key + '.cfg'
            return_path = path.join(p, fname)
            if path.exists(return_path): return return_path
        return None

config_path = ConfigPath()



#!/usr/bin/env python
from os import path
from random import choice
from string import letters, digits

from sh import git
from tornado import autoreload
from tornado.ioloop import IOLoop
from tornado.options import define, options, parse_config_file
from tornado.web import Application

from config import config_path
from handler import urls

class AttrDict(dict):
    def __getattr__(self, key):
        return self[key]
    def __setattr__(self, key, val):
        self[key] = val

def _rand_str(length=64):
    return ''.join([choice(letters + digits) for i in range(0, length)])

def make_app(config=None):
    root = path.dirname(__file__)
    static_path = path.join(root, 'static')
    template_path = path.join(root, 'template')

    app_config = dict(static_path=static_path,
                      template_path=template_path,
                      login_url='/login')

    define('port', default='8080', type=int)
    define('address', default='localhost', type=str)
    define('testing', default=False, type=bool)
    define('repo', default=None, type=str)
    define('username', default=None, type=str)
    define('pwdhash', default=None, type=str)
    define('autosave', default=False, type=bool)

    if config is not None:
        # This should only ever be used for testing
        parse_config_file(config)
    else:
        parse_config_file(config_path.web)

    if options.testing:
        app_config.update(debug=True)

    server = Application(urls, **app_config)
    server.settings = AttrDict(server.settings)
    server.settings.repo = options.repo
    server.settings.username = options.username
    server.settings.pwdhash = options.pwdhash
    server.settings.session = _rand_str()
    server.settings.config_path = config_path
    server.settings.autosave = options.autosave

    server.git = git.bake(_cwd=server.settings.repo)

    return server

def main():
    server = make_app()
    server.listen(options.port, options.address)
    autoreload.start()
    autoreload.watch(config_path.web)
    IOLoop.instance().start()

if __name__ == '__main__':
    main()



import re
from base64 import b64encode, b64decode
from os import listdir, path

from tornado.web import RequestHandler

class BaseHandler(RequestHandler):
    def render(self, template, **kwargs):
        self.starred_notes = self.get_starred()
        kwargs.update(starred_notes=self.starred_notes)

        notebook_name = kwargs.get('notebook_name', None)
        if notebook_name is not None:
            kwargs['notes'] = self._notes_list(notebook_name)
        else:
            kwargs['notebook_name'] = ''
            kwargs['notes'] = []

        kwargs['notebooks'] = self._notebooks_list(kwargs.get('hide_notebooks',
                                                              False))
        super(BaseHandler, self).render(template, **kwargs)

    def _notebooks_list(self, hide_notebooks=False):
        if self.settings.repo is None or not path.isdir(self.settings.repo) or \
        hide_notebooks:
            return []

        return sorted([self.decode_name(nb) for nb in \
                       listdir(self.settings.repo) if nb != ''])

    def _notes_list(self, notebook_name):
        notes_path = path.join(self.settings.repo,
                               self.encode_name(notebook_name))

        all_notes = [n for n in listdir(notes_path) if not \
                     n.startswith('.')]
        starred = []
        unstarred = []

        while all_notes:
            note = self.decode_name(all_notes.pop())
            if u'%s/%s' % (notebook_name, note) in self.starred_notes:
                starred.append(note)
            else:
                unstarred.append(note)

        return sorted(starred) + sorted(unstarred)

    def highlight(self, text, highlight):
        return text.replace(highlight.encode('utf8'),
                            "<font style=background-color:yellow>%s</font>" % \
                            highlight.encode('utf8'))

    def get_current_user(self):
        if self.settings.username is None and self.settings.pwdhash is None \
        or self.settings.username == '' and self.settings.pwdhash == '':
            return True

        return self.get_cookie('session', '') == self.settings.session

    def encode_name(self, name):
        return re.sub(r'[:\|\\\/\?\*"]', lambda m: self._xmlescape(m), name.replace('+', ' '))

    def _xmlescape(self, value):
        return '&#' + str(ord(value.group())) + ';'
        

    def _xmlunescape(self, value):
        return chr(int(value.group(1)))

    def decode_name(self, name):
        return re.sub(r'&#(\d+);', lambda m: self._xmlunescape(m), name) 

    def get_starred(self):
        starred_list = self.get_cookie('starred_notes')
        if starred_list is None:
            return []
        return b64decode(starred_list).decode('utf8').split(',')
